Group {
  Core  = Region[1000] ;
  Coil  = Region[1001] ;
  Sur0  = Region[1010] ;

  DomainCC_Mag = Region[ {Coil} ];
  DomainC_Mag  = Region[ {Core} ];
  DomainB_Mag  = Region[ {} ];
  DomainS_Mag  = Region[ {Coil} ];
  Domain_Mag   = Region[ {DomainCC_Mag, DomainC_Mag} ];
  DomainNL     = Region[ {} ];
  
}

Include "Jacobian_Lib.pro"
Include "Integration_Lib.pro"

Function {
  mu0 = 4e-7*Pi;

  k[ Core ] =  209 ;
  k[ Coil]  =  394 ;

  rhoc  [ Core ] = 8871 * 261 ;
  rhoc  [ Coil ] = 8900 * 385 ;

  // nu [ DomainNL ] = nu_core[$1] ;
  // dhdb_NL [ DomainNL ] = dhdb_core_NL[$1];
  // dhdb [ DomainNL ] = dhdb_core[$1];
  nu [ Core ] = 1./(mu0*800);
  nu [ Coil ] = 1./mu0;

  sigma [ Core ] = 5e7;
  
  JS[] = 1.e6;

  t0 = 0.;
  t1 = 50.;
  dt = 10.;
  
  AmbT[]=273;
  SaveFct[] = !($TimeStep % 1) ; 
  
  Freq = 500.;
  w = 2*Pi*Freq;
}

Constraint {
  { Name Sta_T ;
    Case {
      { Region Domain_Mag ; Type Init; Value AmbT[] ; }
    }
  }
  { Name MagneticVectorPotential_2D;
    Case {
      { Region Sur0 ; Value 0.; }
    }
  }
 
  
  { Name SourceCurrentDensityZ;
    Case {
	  { Region DomainS_Mag; Value JS[]; }
    }
  }
}

FunctionSpace {
  // Magnetic vector potential a (b = curl a)
  { Name Hcurl_a_Mag_2D; Type Form1P;
    BasisFunction {
      { Name se; NameOfCoef ae; Function BF_PerpendicularEdge;
        Support Domain_Mag; Entity NodesOf[ All ]; }
    }
    Constraint {
      { NameOfCoef ae; EntityType NodesOf;
        NameOfConstraint MagneticVectorPotential_2D; }
    }
  }

  // Gradient of Electric scalar potential (2D)
  { Name Hregion_u_Mag_2D; Type Form1P;
    BasisFunction {
      { Name sr; NameOfCoef ur; Function BF_RegionZ;
        Support DomainC_Mag; Entity DomainC_Mag; }
    }
    GlobalQuantity {
      { Name U; Type AliasOf       ; NameOfCoef ur; }
      { Name I; Type AssociatedWith; NameOfCoef ur; }
    }
    Constraint {
      { NameOfCoef U; EntityType Region;
        NameOfConstraint Voltage_2D; }
      { NameOfCoef I; EntityType Region;
        NameOfConstraint Current_2D; }
    }
  }

  
  // Source current density js (fully fixed space)
  { Name Hregion_j_Mag_2D; Type Vector;
    BasisFunction {
      { Name sr; NameOfCoef jsr; Function BF_RegionZ;
        Support DomainS_Mag; Entity DomainS_Mag; }
    }
    Constraint {
      { NameOfCoef jsr; EntityType Region;
        NameOfConstraint SourceCurrentDensityZ; }
    }
  }
  { Name Hgrad_T; Type Form0; 
    BasisFunction {
      { Name sn; NameOfCoef Tn; Function BF_Node; Support Domain_Mag; 
        Entity NodesOf[All]; }
    }
    Constraint {
      { NameOfCoef Tn; EntityType NodesOf ; NameOfConstraint Sta_T; }
    }
  }

}


Formulation {
  { Name Magnetodynamics_av_2D; Type FemEquation;
    Quantity {
      { Name a ; Type Local ; NameOfSpace Hcurl_a_Mag_2D; }
      { Name ur; Type Local ; NameOfSpace Hregion_u_Mag_2D; }
      { Name I ; Type Global; NameOfSpace Hregion_u_Mag_2D [I]; }
      { Name U ; Type Global; NameOfSpace Hregion_u_Mag_2D [U]; }
      { Name js; Type Local ; NameOfSpace Hregion_j_Mag_2D; }
    }
    Equation {
      Galerkin { [ nu[{d a}] * Dof{d a} , {d a} ]; In Domain_Mag;
                 Jacobian Vol; Integration CurlCurl; }
				 
      // Galerkin { JacNL [ dhdb_NL[{d a}] * Dof{d a} , {d a} ] ;
				// In DomainNL ; Jacobian Vol ; Integration I1 ; }

      Galerkin { DtDof [ sigma[] * Dof{a} , {a} ]; In DomainC_Mag;
                 Jacobian Vol; Integration CurlCurl; }
				 
      Galerkin { [ sigma[] * Dof{ur} , {a} ]; In DomainC_Mag;
                 Jacobian Vol; Integration CurlCurl; }

      Galerkin { [ - Dof{js} , {a} ]; In DomainS_Mag;
                 Jacobian Vol;
                 Integration CurlCurl; }

      Galerkin { DtDof [ sigma[] * Dof{a} , {ur} ]; In DomainC_Mag;
                 Jacobian Vol; Integration CurlCurl; }
      Galerkin { [ sigma[] * Dof{ur} , {ur} ]; In DomainC_Mag;
                 Jacobian Vol; Integration CurlCurl; }
          }
    }

  { Name TheDyn ; Type FemEquation; 
    Quantity { 
      { Name T;  Type Local; NameOfSpace Hgrad_T; }
	  { Name a ; Type Local ; NameOfSpace Hcurl_a_Mag_2D; }
      { Name ur; Type Local ; NameOfSpace Hregion_u_Mag_2D; }
    }
    Equation {
      Galerkin { [ k[] * Dof{d T} , {d T} ]; 
                 In Domain_Mag; Integration I1; Jacobian Vol;  }

      Galerkin { DtDof [ rhoc[] * Dof{T} , {T} ]; 
                 In Domain_Mag; Integration I1; Jacobian Vol;  }

	  Galerkin { [ -sigma[]*((Re[{ur}]-Im[w*{a}])*(Re[{ur}]-Im[w*{a}]) + (Im[{ur}]+Re[w*{a}])*(Im[{ur}]+Re[w*{a}]))  , {T} ];
	             In DomainC_Mag; Integration CurlCurl; Jacobian Vol;  }
    }
  }

}


Resolution {
  { Name TheDyn;
    System {
      { Name T; NameOfFormulation TheDyn; }
      { Name B; NameOfFormulation Magnetodynamics_av_2D;
         Type ComplexValue; Frequency Freq;}
    }
    Operation {
      InitSolution[B];
      // IterativeLoop[Nb_max_iter, stop_criterion, relaxation_factor]{GenerateJac[B] ; SolveJac[B] ; }
	  Generate[B];Solve[B];
	  SaveSolution[B];
      InitSolution[T]; SaveSolution[T];
      TimeLoopTheta[t0, t1, dt, 1.] {
	// IterativeLoop[NL_NbrMax, NL_Eps, NL_Relax] {
	  // GenerateJac[T]; SolveJac[T];}
	Generate[T]; Solve[T];
	SaveSolution[T];
	}
	}
	}

}


PostProcessing {
  { Name MagDyn_av_2D; NameOfFormulation Magnetodynamics_av_2D;
    Quantity {
      { Name a; 
        Value { 
          Local { [ {a} ]; In Domain_Mag; Jacobian Vol; } 
        } 
      }
      { Name az; 
        Value { 
          Local { [ CompZ[{a}] ]; In Domain_Mag; Jacobian Vol; }
        }
      }
      { Name b; 
        Value { 
          Local { [ {d a} ]; In Domain_Mag; Jacobian Vol; }
        }
      }
      { Name h; 
        Value { 
          Local { [ nu[{d a}] * {d a} ]; In Domain_Mag; Jacobian Vol; }
        }
      }
      { Name j; 
        Value { 
          Local { [ - sigma[]*(Dt[{a}]+{ur}) ]; In DomainC_Mag; 
                  Jacobian Vol; }
		  Local { [ {js} ]; In DomainS_Mag; 
                  Jacobian Vol; } 
        } 
      }
      { Name jz; 
        Value { 
          Local { [ - sigma[]*CompZ[Dt[{a}]+{ur}] ]; In DomainC_Mag; 
                  Jacobian Vol; } 
          Local { [ CompZ[{js}] ]; In DomainS_Mag; 
                  Jacobian Vol; }
        } 
      }
      { Name roj2;
        Value { 
          Local { [ sigma[]*SquNorm[Dt[{a}]+{ur}] ]; In DomainC_Mag;
                  Jacobian Vol; }
          Local { [ SquNorm[{js}]/sigma[] ]; In DomainS_Mag;
                  Jacobian Vol; }
        } 
      }
	  { Name roj2b;
        Value { 
          Local { [sigma[]*((Re[{ur}]-Im[w*{a}])*(Re[{ur}]-Im[w*{a}]) + (Im[{ur}]+Re[w*{a}])*(Im[{ur}]+Re[w*{a}])) ]; In DomainC_Mag; 
                  Jacobian Vol; } 
        } 
      }
      { Name U; Value { Local { [ {U} ]; In DomainC_Mag; } } }
      { Name I; Value { Local { [ {I} ]; In DomainC_Mag; } } }
	  { Name PJoules; 
        Value {
          Integral { [ sigma[]*SquNorm[Dt[{a}]+{ur}] * 2 * Pi ];
		  In DomainC_Mag; Integration I1; Jacobian Vol; }
		  Integral { [ SquNorm[{js}]/sigma[] * 2 * Pi ];
		  In DomainS_Mag; Integration I1; Jacobian Vol; }
		 }
	  }
    }
  }
  
  
  { Name TheDyn; NameOfFormulation TheDyn;
    Quantity {
      { Name T; Value{ Local{ [ {T} - 273 ] ;
	    In Domain_Mag; Jacobian Vol; } } }
	  { Name q; Value{ Local{ [ -k[]*{d T} ] ;
	    In Domain_Mag; Jacobian Vol; } } }
	  { Name qv; Value{ 
	    Local{ [ sigma[]*((Re[{ur}]-Im[w*{a}])*(Re[{ur}]-Im[w*{a}]) + (Im[{ur}]+Re[w*{a}])*(Im[{ur}]+Re[w*{a}])) ]; In DomainC_Mag; Jacobian Vol; } 
		} }
	  { Name Qt; 
        Value {
          Integral { [ sigma[]*((Re[{ur}]-Im[w*{a}])*(Re[{ur}]-Im[w*{a}]) + (Im[{ur}]+Re[w*{a}])*(Im[{ur}]+Re[w*{a}])) * 2 * Pi ];
		  In DomainC_Mag; Integration I1; Jacobian Vol; }
		  }
		}
    }
  }
}



PostOperation {
        
    { Name Ther ; NameOfPostProcessing TheDyn ;
      Operation {
        Print [ T, OnElementsOf Domain_Mag, File "therm_T.pos" ];
		Print [ qv, OnElementsOf Domain_Mag, File "therm_qv.pos" ];
      }
    }    
    { Name Mag ; NameOfPostProcessing MagDyn_av_2D ;
      Operation {
        Print [ roj2b, OnElementsOf Domain_Mag, File "therm_Pjoules.pos" ];
		Print [ jz, OnElementsOf Domain_Mag, File "therm_jz.pos" ];
		Print [ b, OnElementsOf Domain_Mag, File "therm_b.pos" ];

      }
    }
  

}